
/******************************************************************************
 * @file     main.c
 * @brief    Nano1xx GPIO Driver Sample code
 * @version  1.0.1
 * @date     04, September, 2012
 *
 * @note
 * Copyright (C) 2012-2014 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#include <stdio.h>
#include <stdint.h>
#include "nano1xx.h"
#include "nano1xx_gpio.h"


/*offset 10 is LPDU*/
char FM0_Frame[512] = {
	0xFF, 0xFF, 0x00, 0x00, 0x7E,
	0xFF, 0xFF, 0xFF, 0xFF, /*broadcast MAC address*/
	0x50, /*MAC control field*/
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10, /*LPDU*/
	0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F, 0x20, /*LPDU*/
	0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F, 0x30, /*LPDU*/
	0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F, 0x40, /*LPDU*/
	0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 0x50, /*LPDU*/
	0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x5B, 0x5C, 0x5D, 0x5E, 0x5F, 0x60, /*LPDU*/
	0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70, /*LPDU*/
	0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x7B, 0x7C, 0x7D, 0x7E, 0x7F, 0x80, /*LPDU*/
	0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x8A, 0x8B, 0x8C, 0x8D, 0x8E, 0x8F, 0x90, /*LPDU*/
	0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9A, 0x9B, 0x9C, 0x9D, 0x9E, 0x9F, 0xA0, /*LPDU*/
	0xA1, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xAB, 0xAC, 0xAD, 0xAE, 0xAF, 0xB0, /*LPDU*/
	0xB1, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xBB, 0xBC, 0xBD, 0xBE, 0xBF, 0xC0, /*LPDU*/
	0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF, 0xD0, /*LPDU*/
	0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xDB, 0xDC, 0xDD, 0xDE, 0xDF, 0xE0, /*LPDU*/
	0xE1, 0xE2, 0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9, 0xEA, 0xEB, 0xEC, 0xED, 0xEE, 0xEF, 0xF0, /*LPDU*/
	0xF1, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8, 0xF9, 0xFA, 0xFB, 0xFC, 0xFD, 0xFE, 0xFF, 0xFF, /*LPDU*/
	0x0A, /*FCS*/
	0x7E, /*Frame End Flag*/
};

void SysInit()
{
	/* Enable system clock source, HIRC and LIRC are default enabled */	
  	UNLOCKREG();
  	CLK->PWRCTL |= (CLK_PWRCTL_HXT_EN | CLK_PWRCTL_LXT_EN);	   
  	LOCKREG();
	while((CLK->CLKSTATUS & CLK_CLKSTATUS_HXT_STB) == 0);
#if 1
  	/* Enable PLL out to 96MHz */
  	CLK->PLLCTL = (CLK_PLLCTL_PLLSRC_HIRC | PLL_IN_12M_OUT_96M);
	while((CLK->CLKSTATUS & CLK_CLKSTATUS_PLL_STB) == 0);

	CLK->CLKDIV0 = (CLK->CLKDIV0 & ~CLK_CLKDIV0_HCLK_MASK) | 2; /* divider is 3 */
#endif
	UNLOCKREG();
    CLK->CLKSEL0 = (CLK->CLKSEL0 & ~CLK_CLKSEL0_HCLK_MASK) | CLK_CLKSEL0_HCLK_PLL;	 /* HCLK = 32MHz */
	//CLK->CLKSEL0 = (CLK->CLKSEL0 & ~CLK_CLKSEL0_HCLK_MASK) | CLK_CLKSEL0_HCLK_HIRC;	 /* HCLK = 32MHz */
    LOCKREG();
	/* Update CPU Clock Frequency */
  	SystemCoreClockUpdate();
}

void TimerInit()
{
	//CLK->CLKSEL1 = (CLK->CLKSEL1 & ~CLK_CLKSEL1_TMR0_MASK) | CLK_CLKSEL1_TMR0_HIRC;
	CLK->CLKSEL1 = (CLK->CLKSEL1 & ~CLK_CLKSEL1_TMR0_MASK) | CLK_CLKSEL1_TMR0_HXT;
	TIMER_Init(TIMER0, 0, 47, TIMER_CTL_MODESEL_ONESHOT);/*no scaling, TCMPR = 47 TIMER_CTL_MODESEL_PERIODIC*/	
	TIMER0->ISR = TIMER_ISR_TMRIS;
}

/*unit = 1/12M*/
#if 0

//3.9us , 1.95us
#define TimerDelayOne()		TimerDelay(16)//46.8
#define TimerDelayHalf()	TimerDelay(1)

__INLINE void TimerDelay(int cnt)
{
	TIMER0->ISR = TIMER_ISR_TMRIS;//clear interrupt flag
	TIMER0->CMPR = cnt;
	TIMER0->CTL |= TIMER_CTL_TMREN;
	while((TIMER0->ISR & TIMER_ISR_TMRIS) == 0);//wait timeout
}
#else
/*32M: 125*/
/*12M: 46.8*/
/*3.9us*/
__ASM __INLINE void TimerDelayOne()
{
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	//NOP;
	//NOP;		
	
	//MOV r8, r8
	BX lr
}
/*
	7 NOP 716ns
	add 1 NOP , 60ns
	1.95us: 
*/
__ASM __INLINE void TimerDelayHalf()
{
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	NOP;
	//NOP;
	//NOP;	
		
	BX lr
}
#endif

/*256Kbps LSB first, but FCS MSB first, idle level is low*/
void SendFM0(char *buf, int len)
{
	int i, j, OneCnt, BitCnt;
	
	OneCnt = 0;
	BitCnt = 0;
	
	for(i = 0; i < len; i++)
	{		
		for(j = 0; j < 8; j++)
		{
			PA0 = ~PA0;
			BitCnt++;

			if(buf[i]&(1<<j))//'1'
			{
				TimerDelayOne();				
				
				if((i >= 5) && (i < len-1))
					OneCnt++;
				
				if(OneCnt >= 5)
				{
					PA0 = ~PA0;
					TimerDelayHalf();
					PA0 = ~PA0;
					TimerDelayHalf();
					OneCnt = 0;
					BitCnt++;
				}					
			}
			else
			{
				TimerDelayHalf();
				PA0 = ~PA0;
				TimerDelayHalf();
				OneCnt = 0;
			}
		}
	}
	
	j = BitCnt % 8;
	
	//pad zero
	if(j)
	{
		j = 8 - j;
		for(i = 0; i < j; i++)
		{
			PA0 = ~PA0;
			TimerDelayHalf();
			PA0 = ~PA0;
			TimerDelayHalf();
		}
	}
	PA0 = ~PA0;
}


/**   
  * @brief  main, GPIO sample code.
  * @param  None.
  * @retval None.
  */
int main (void)
{
    int32_t i32Err;

    SysInit();
    TimerInit();
	   
    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure Bit0 in GPIOA to Output pin and Bit1 in GPIOA to Input pin then close it                  */
    /*-----------------------------------------------------------------------------------------------------*/
	PA0 = 0;//idle is low
	GPIO_Open(GPIOA, GPIO_PMD_PMD0_OUTPUT, GPIO_PMD_PMD0_MASK);
	GPIO_Open(GPIOA, GPIO_PMD_PMD1_INPUT, GPIO_PMD_PMD1_MASK);
	    
	
#if 0	
	PA0 = ~PA0;

TimerDelayOne();
PA0 = ~PA0;
TimerDelayOne();

#endif
	
	SendFM0(FM0_Frame, 92+64 + 128);
}

#ifdef USE_ASSERT
/**
  * @brief  The function prints the source file name and line number where the assert_param() error
  *         occurs, and then stops in an infinite loop. User can add his own codes here if necessary.
  * @param[in] file Source file name
  * @param[in] line Line number
  * @return None
  */
void assert_error(uint8_t * file, uint32_t line)
{
 	GCR->PB_L_MFP = (GCR->PB_L_MFP & ~0x77) | (PB1_MFP_UART0_TX | PB0_MFP_UART0_RX);  /* Select multi-function pin for UART0 */
 	CLK->APBCLK |= CLK_APBCLK_UART0_EN;    /* Enable UART0 clock */
	CLK->CLKDIV0 &= ~CLK_CLKDIV0_UART_MASK;
	CLK->CLKSEL1 = (CLK->CLKSEL1 & ~CLK_CLKSEL1_UART_MASK) | CLK_CLKSEL1_UART_HXT;  /* Select 12 Mhz XTAL */

 
    /* Set UART to 115200,n,8,1,none */
    UART0->BAUD = 0x67;             /* Baud Rate:115200 for 12MHz */
    UART0->TLCTL = 0x03;            /* Word len is 8 bits */          
	
	printf("[%s] line %d : wrong parameters.\r\n", file, line);

	/* Infinite loop */
	while(1) ;

}
#endif

/*** (C) COPYRIGHT 2012 Nuvoton Technology Corp. ***/


