/******************************************************************************
 * @file     main.c
 * @brief    Nano1xx TIMER Driver Sample code
 * @version  1.0.1
 * @date     04, September, 2012
 *
 * @note
 * Copyright (C) 2012-2014 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#include <stdio.h>
#include <stdbool.h>
#include "nano1xx.h"
#include "nano1xx_timer.h"
#include "nano1xx_gpio.h"
#include "nano1xx_assert.h"

__align(4) uint8_t FM0ReceiveBuf[512];

uint32_t g_capdata[2048], g_head;



void SysInit()
{
	/* Enable system clock source, HIRC and LIRC are default enabled */	
  	UNLOCKREG();
  	CLK->PWRCTL |= (CLK_PWRCTL_HXT_EN | CLK_PWRCTL_LXT_EN);	   
  	LOCKREG();
	while((CLK->CLKSTATUS & CLK_CLKSTATUS_HXT_STB) == 0);
#if 0
  	/* Enable PLL out to 96MHz */
  	CLK->PLLCTL = (CLK_PLLCTL_PLLSRC_HIRC | PLL_IN_12M_OUT_96M);
	while((CLK->CLKSTATUS & CLK_CLKSTATUS_PLL_STB) == 0);

	CLK->CLKDIV0 = (CLK->CLKDIV0 & ~CLK_CLKDIV0_HCLK_MASK) | 2; /* divider is 3 */
#endif
	UNLOCKREG();
    //CLK->CLKSEL0 = (CLK->CLKSEL0 & ~CLK_CLKSEL0_HCLK_MASK) | CLK_CLKSEL0_HCLK_PLL;	 /* HCLK = 32MHz */
	CLK->CLKSEL0 = (CLK->CLKSEL0 & ~CLK_CLKSEL0_HCLK_MASK) | CLK_CLKSEL0_HCLK_HIRC;	 /* HCLK = 32MHz */
    LOCKREG();
	/* Update CPU Clock Frequency */
  	SystemCoreClockUpdate();
}

void TimerInit()
{
	/*TC0 using PA.12*/
	GCR->PA_H_MFP = GCR->PA_H_MFP &~(PA12_MFP_MASK) | PA12_MFP_TMR0_CAP;
	
	CLK->CLKSEL1 = (CLK->CLKSEL1 & ~CLK_CLKSEL1_TMR0_MASK) | CLK_CLKSEL1_TMR0_HXT;
	CLK->APBCLK |= CLK_APBCLK_TMR0_EN;
	TIMER0->CTL = 0;
	TIMER0->ISR = (TIMER_ISR_TMRIS | TIMER_ISR_TCAPIS | TIMER_ISR_TMRWAKESTS | TIMER_ISR_NCAPDETSTS);
	TIMER0->PRECNT = 0;
	TIMER0->CMPR = 0xFFFFFF;
	
	/*trigger counting , rising-rising*/
	TIMER0->CTL = /*TIMER_CTL_CAPCNTMOD_TRIGGER_CNT*/TIMER_CTL_CAPCNTMOD_FREE_CNT | TIMER_CTL_TCAPEDGE_RR | TIMER_CTL_TCAPEN /*| TIMER_CTL_TCAPDEBEN*/
					| TIMER_CTL_MODESEL_PERIODIC;
	
	TIMER0->CTL |= TIMER_CTL_TMREN;				
}

__ASM __INLINE void TimerDelayOne()
{
	NOP;
	BX lr;		
	//MOV r8, r8
}		

void HardFault_Handler()
{
	TimerDelayOne();
	TimerDelayOne();
	while(1);
}

BOOL ReceiveFM0()
{
	register uint8_t BitCnt, OneCnt, FrontHalf;
	register uint32_t data;
	uint32_t FrameFlag, base, CapData, index;
	uint8_t data2, discardZero;
	
	BitCnt = 0;
    OneCnt = 0;
    index = 0;
    FrameFlag = 0; 
    FrontHalf = 0;
    data = 0;
    discardZero = 0;
    
    
    
    /*get begin time*/
    while((TIMER0->ISR & TIMER_ISR_TCAPIS) == 0);
    base = TIMER0->TCAP;
    TIMER0->ISR = TIMER_ISR_TCAPIS;
    
    PA2 = 1;
    
    while(1)
   	{
    	if(TIMER0->ISR & TIMER_ISR_TCAPIS)
		{			
			 CapData = TIMER0->TCAP - base;
			 base = TIMER0->TCAP;

			g_capdata[g_head++] = CapData;
			
			//if(g_head == 4)//>= 1024)
			if(	(TIMER0->ISR & TIMER_ISR_NCAPDETSTS) || (g_head >= 2048))
			{
				TimerDelayOne();
				TimerDelayOne();
				while(1);
			}
			
			TIMER0->ISR = TIMER_ISR_TCAPIS;
			
			if(CapData > 84)//7us two '1'
			{				
				data |= (3<<BitCnt);
				BitCnt += 2;
				OneCnt += 2;

				
			}
			else if(CapData >= 60)//5us one '1' + one '0' maybe
			{						
				data |= (1<<BitCnt);
				BitCnt++;
				OneCnt++;
				if(FrontHalf == 0)
				{
					FrontHalf = 1;
					/*eliminate zero*/
					if((OneCnt == 5) && (FrameFlag))
					{
						OneCnt = 0;
						discardZero = 1;
					}
					else
					{
						OneCnt = 0;
						BitCnt++;
					}
				}
				else
					FrontHalf = 0;			
			}
			else if(CapData < 60)//4us one '0'
			{					
				if((OneCnt == 5) && (FrameFlag))
				{
					OneCnt = 0;
					discardZero = 1;
				}
				else
				{
					OneCnt = 0;
					BitCnt++;				
				}
			}
						
			if(BitCnt >= 8)
			{
				data2 = data & 0xFF;
				
				if(BitCnt > 8)
				{					
					data = data >> 8;
				}
				else
				{					
					data = 0;
				}
					
				FM0ReceiveBuf[index++] = data2;				
				BitCnt -= 8;							

				if(index >= 512)
				{
					TimerDelayOne();
					TimerDelayOne();
					while(1);					
				}
				
				if(data2 == 0x7E)
				{
					OneCnt = 0;
					
					if(discardZero == 0)
						FrameFlag++;
					if(FrameFlag == 2)
						break;
				}
				else if(OneCnt >= 6)
				{
					if(FrameFlag)				
						return FALSE;//error
				}
				
				discardZero = 0;

			}
			if(OneCnt >= 7)
			{
				if(FrameFlag)				
					return FALSE;//error
			}
			
		}		
	}
	PA2 = 0;
	
	return TRUE;
}

int main (void)
{	
    SystemInit();
    TimerInit();
    
    PA2 = 0;
    GPIO_Open(GPIOA, GPIO_PMD_PMD2_OUTPUT, GPIO_PMD_PMD2_MASK);
    
    ReceiveFM0();
    
    while(1);
	
    return 0;
}


#ifdef USE_ASSERT
/**
  * @brief  The function prints the source file name and line number where the assert_param() error
  *         occurs, and then stops in an infinite loop. User can add his own codes here if necessary.
  * @param[in] file Source file name
  * @param[in] line Line number
  * @return None
  */
void assert_error(uint8_t * file, uint32_t line)
{
 	GCR->PB_L_MFP = (GCR->PB_L_MFP & ~0x77) | (PB1_MFP_UART0_TX | PB0_MFP_UART0_RX);  /* Select multi-function pin for UART0 */
 	CLK->APBCLK |= CLK_APBCLK_UART0_EN;    /* Enable UART0 clock */
	CLK->CLKDIV0 &= ~CLK_CLKDIV0_UART_MASK;
	CLK->CLKSEL1 = (CLK->CLKSEL1 & ~CLK_CLKSEL1_UART_MASK) | CLK_CLKSEL1_UART_HXT;  /* Select 12 Mhz XTAL */

 
    /* Set UART to 115200,n,8,1,none */
    UART0->BAUD = 0x67;             /* Baud Rate:115200 for 12MHz */
    UART0->TLCTL = 0x03;            /* Word len is 8 bits */          
	
	printf("[%s] line %d : wrong parameters.\r\n", file, line);

	/* Infinite loop */
	while(1) ;

}
#endif

/*** (C) COPYRIGHT 2012 Nuvoton Technology Corp. ***/

